#!/usr/bin/env python
# coding: utf-8

# <figure>
#  <img src="./images/123H.jpg"   style="width:100px;height:100px;">
# </figure>

# In[1]:


import math
import numpy as np
import matplotlib.pyplot as plt
import math 
from numpy.polynomial import Laguerre
import sys


#The lines below help to export to html
import plotly.io as pio
pio.renderers.default='notebook'
#import seaborn as sns


get_ipython().run_line_magic('matplotlib', 'notebook')
get_ipython().run_line_magic('pylab', '')


# In[2]:


"""Constants parameters"""
"""M: molar mass in g/mol
    I: spin of nucleus
    T: temperature in K
    p: pressure in Pa
    n: amount in mol
"""
# Natural constants
planck = 6.62607004e-34 # J s
avogadro = 6.022140857e23 # / mol
hbar = 0.001*avogadro*planck/(2.0*math.pi) # kJ s / mol
R = 0.0083145 # kJ / (K mol) 
KB = 1.380649e-26 # in KJ/K , KB is the Boltzmann constant
n = 1 #amount in mol
p = 100000 #in pa which is 1.5 bar
#R = 8.314 #J/ (mol K)

# Nuclide masses: nuclides = ('H', 'D', 'T')
##masses : https://www.translatorscafe.com/unit-converter/zh-HK/molar-mass/?q=H
m_H = 1.00794 # in g/mol
m_D = 2.01588 # in g/mol
m_T = 3.02382 # in g/mol 

m_H2 = 1.00794*2 # in g/mol
m_D2 = 2.01588*2 # in g/mol
m_T2 = 3.02382*2 # in g/mol
m_HD = 1.00794 + 2.01588 # in g/mol
m_HT = 1.00794 + 3.02382 # in g/mol
m_DT = 2.01588 + 3.02382 # in g/mol

##spins : {'H': 0.5,   'D': 1.0,   'T': 1.5}
I_H = 0.5
I_D = 1.0
I_T = 0.5


# # Evaluation of the free $H_2$ and isotopologues

# In[3]:


def partitiongasphase(M, ze_free, ω_free, I, inertia, T, is2D=False,symmetryCoupling=True):
    """Calculates the translational partition function of an ideal gas.
    Model: 'Ideal gas particle in a box'
    M: molar mass in g/mol
    T: temperature in K
    p: pressure in Pa
    n: amount in mol
    
    Calculates the partition of a quantum-statistical 2D rigid rotor.
    
    I: spin of a single nucleus (irrelevant if symmetryCoupling=False)
    T: temperature in K
    inertia: moment of inertia in Å² g/mol
    is1D: False (default) → use 3D rotor model
          True → use 2D rotor model (rotation (hindered in one direction)
    symmetryCoupling:
          True (default) → take coupling with spin into account
                       (homonuclear diatomic molecule)
          False → no coupling with spin (heteronuclear molecules)
    """
    # translational modes model as ideal gas particle in a box
    V_0 = 1000 * n * R * T / p # in m³
    particleInBox = 1e-9 * (M * R * T) / (2 * pi * hbar**2) # / m²
    q_trans = particleInBox**1.5 * V_0
    
    # run 3D rigid rotor
    if is2D:
        Model: 'Rigid rotor (2D)'
    else:
        Model: 'Rigid rotor (3D)'
    q_rigid = 0.0
    q_rigid_3D = 0.0
    Θ_rot_0 = (5.0e25 * hbar**2) / (inertia * R) # in K
    beta = 1/(KB*T)
    for j in range(100):
        if symmetryCoupling:
            add = int((I == int(I)) == (j % 2 == 0))
            spindegen = (I + add) * (2 * I + 1)
        else:
            spindegen = 1
        if is2D:
            contrib = spindegen * exp(- (beta*((hbar**2*j*(j+1))/2*I ))) # For 1D
            contrib_2D = spindegen * exp(- (Θ_rot_0/T)*j*j) #For 2D
            q_rigid +=contrib_2D
        else:
            contrib_3D = spindegen * (2*j + 1) * exp(- (Θ_rot_0 / T) * j * (j+1))
            q_rigid +=contrib_3D
    
    # run vibrational for free 𝐻2 and isotopologues
    E_0 = 0.5*(hbar*ω_free)
    qharm_0 = np.exp(-E_0/(R*T))
    qharm = 1.0/(1.0 - np.exp(-(hbar*ω_free)/(R*T)))
    return q_trans, q_rigid, qharm*qharm_0


#T=np.linspace(10,300,30)
T=np.linspace(80,300,23)
light = 29979245800 # cm/s
per_cm_to_ang_freq = 2*math.pi*light # cm-1 to Hz

ω_free_HH =  4526.27 * per_cm_to_ang_freq
ω_free_DD =  3206.78 * per_cm_to_ang_freq
ω_free_TT =  2616.46 * per_cm_to_ang_freq
ω_free_HD =  3922.41 * per_cm_to_ang_freq
ω_free_HT =  3696.82 * per_cm_to_ang_freq
ω_free_DT =  2926.54 * per_cm_to_ang_freq


ze_free_HH = 0.737
inertia_HH = (m_H2)*(ze_free_HH)**2
inertia_DD = (m_D2)*(ze_free_HH)**2
inertia_TT = (m_T2)*(ze_free_HH)**2
inertia_HD = (m_HD)*(ze_free_HH)**2
inertia_HT = (m_HT)*(ze_free_HH)**2
inertia_DT = (m_DT)*(ze_free_HH)**2

gasphase_H2 = partitiongasphase(m_H2, ze_free_HH, ω_free_HH, I_H, inertia_HH, T, is2D=False,symmetryCoupling=True)
gasphase_D2 = partitiongasphase(m_D2, ze_free_HH, ω_free_DD, I_D, inertia_DD, T, is2D=False,symmetryCoupling=True)
gasphase_T2 = partitiongasphase(m_T2, ze_free_HH, ω_free_TT, I_T, inertia_TT, T, is2D=False,symmetryCoupling=True)

gasphase_HD = partitiongasphase(m_HD, ze_free_HH, ω_free_HD, I_H, inertia_HD, T, is2D=False,symmetryCoupling=False)
gasphase_HT = partitiongasphase(m_HT, ze_free_HH, ω_free_HT, I_H,  inertia_HT, T, is2D=False,symmetryCoupling=False)
gasphase_DT = partitiongasphase(m_DT, ze_free_HH, ω_free_DT, I_H, inertia_DT, T, is2D=False,symmetryCoupling=False)


# # Model: One 𝐻2 and isotopologues attached to the $𝐶𝑢^{+}−(𝐻_{2}𝑂)(H_{2})$ when $𝐶𝑢^{+}−(𝐻_{2}𝑂)(H_{2})$ is frozen

# In[4]:


def partitionmodel(ω_hind_rot, ω_harm, ω_harm_Cu, ω_ax, ω_lat, ze_harm, inertia_harm, I, T, is2D=True,symmetryCoupling=True):
    """Calculates the partition function of a vibrational mode
	described as harmonic oscillator.
	
	mu: reduced mass in g/mol
	k: force constant in kJ/(mol Å²)
	T: temperatur in K
    """
    
     # run 2D rigid rotor
    if is2D:
        Model: 'Rigid rotor (2D)'
    else:
        Model: 'Rigid rotor (3D)'
    q_harm_rigid = 0.0
    Θ_rot_0 = (5.0e25 * hbar**2) / (inertia_harm * R) # in K
    beta = 1/(KB*T)
    for j in range(100):
        if symmetryCoupling:
            add = int((I == int(I)) == (j % 2 == 0))
            spindegen = (I + add) * (2 * I + 1)
        else:
            spindegen = 1
        if is2D:
            #contrib = spindegen * exp(- (beta*((hbar**2*j*(j+1))/2*I ))) # For 1D
            contrib_2D = spindegen * exp(- (Θ_rot_0/T)*j*j) #For 2D
            q_harm_rigid +=contrib_2D
        else:
            contrib_3D = spindegen * (2*j + 1) * exp(- (Θ_rot_0 / T) * j * (j+1))
            q_harm_rigid +=contrib_3D
    
     # Hindered rotation 
    ΔE = hbar * ω_hind_rot #in 'kJ/mol'
    E_0 =  0.5 * ΔE #in 'kJ/mol'
    ex = ΔE/(R*T)
    q_hind_0 = np.exp(-0.5*ex)
    q_hind = 1.0/(1.0 - np.exp(-ex))
    
    # Hindered Axial translational modes
    A_ax = np.exp(-(hbar*ω_ax)/(2*R*T))
    B_ax = (1.0-np.exp(-(hbar*ω_ax/(R*T))))
    q_trans_ax = A_ax/B_ax
    
    # Hindered Axial translational modes
    A_lat = np.exp(-(hbar*ω_lat)/(2*R*T))
    B_lat = (1.0-np.exp(-(hbar*ω_lat/(R*T))))
    q_trans_lat = A_lat/B_lat
    
    # Vibrational modes for adsorbed 𝐻2 and isotopologues
    E_vib_0 = 0.5*(hbar*ω_harm)
    q_vib_0 = np.exp(-E_vib_0/(R*T))
    q_vib = 1.0/(1.0 - np.exp(-(hbar*ω_harm)/(R*T)))
    
    # Vibrational modes for adsorbed 𝐻2 and isotopologues
    E_vib_Cu_0 = 0.5*(hbar*ω_harm_Cu)
    q_vib_Cu_0 = np.exp(-E_vib_Cu_0/(R*T))
    q_vib_Cu = 1.0/(1.0 - np.exp(-(hbar*ω_harm_Cu)/(R*T)))
   
    return q_harm_rigid, q_hind*q_hind_0, q_trans_ax, q_trans_lat, q_vib*q_vib_0, q_vib_Cu*q_vib_Cu_0



ze_harm_HH = 0.789
inertia_harm_HH = m_H2*(ze_harm_HH)**2
inertia_harm_DD = m_D2*(ze_harm_HH)**2
inertia_harm_TT = m_T2*(ze_harm_HH)**2
inertia_harm_HD = m_HD*(ze_harm_HH)**2
inertia_harm_HT = m_HT*(ze_harm_HH)**2
inertia_harm_DT = m_DT*(ze_harm_HH)**2

#Frequenncies in cm-1 obtained when Cu+(H2O)(H2) is frozen
ω_hind_rot_HH = (1307.03) * per_cm_to_ang_freq
ω_hind_rot_DD =  (926.01) * per_cm_to_ang_freq
ω_hind_rot_TT =  (755.54) * per_cm_to_ang_freq

ω_ax_HH =  (280.07) * per_cm_to_ang_freq   
ω_lat_HH = (198.36) * per_cm_to_ang_freq   
ω_ax_DD =  (198.42)  * per_cm_to_ang_freq  
ω_lat_DD = (140.53) * per_cm_to_ang_freq   
ω_ax_TT =  (161.90) * per_cm_to_ang_freq    
ω_lat_TT = (114.66) * per_cm_to_ang_freq


ω_harm_HH = (3754.23) * per_cm_to_ang_freq
ω_harm_DD =  (2659.81) * per_cm_to_ang_freq
ω_harm_TT =  (2170.17) * per_cm_to_ang_freq


ω_harm_CuHH = (871.05) * per_cm_to_ang_freq
ω_harm_CuDD =  (617.12) * per_cm_to_ang_freq
ω_harm_CuTT =  (503.52) * per_cm_to_ang_freq


# frequencies obtained in case the Hybrids (HD, HT, DT) attached to the Cu+(H2O)(H2) when Cu+(H2O)(H2) is frozen
ω_hind_rot_HD =  (1155.10) * per_cm_to_ang_freq
ω_hind_rot_HT =  (1116.51) * per_cm_to_ang_freq
ω_hind_rot_DT =  (849.65) * per_cm_to_ang_freq

ω_ax_HD =  (228.48) * per_cm_to_ang_freq   
ω_lat_HD = (158.35) * per_cm_to_ang_freq   
ω_ax_HT =  (197.45)  * per_cm_to_ang_freq  
ω_lat_HT = (134.10) * per_cm_to_ang_freq   
ω_ax_DT =  (177.21) * per_cm_to_ang_freq    
ω_lat_DT = (124.32) * per_cm_to_ang_freq


ω_harm_HD =  (3260.05) * per_cm_to_ang_freq
ω_harm_HT =  (3078.15) * per_cm_to_ang_freq
ω_harm_DT =  (2429.77) * per_cm_to_ang_freq


ω_harm_CuHD =  (698.35) * per_cm_to_ang_freq
ω_harm_CuHT =  (589.45) * per_cm_to_ang_freq
ω_harm_CuDT =  (548.82) * per_cm_to_ang_freq



ads_model_H2 = partitionmodel(ω_hind_rot_HH, ω_harm_HH, ω_harm_CuHH, ω_ax_HH, ω_lat_HH, ze_harm_HH, inertia_harm_HH, I_H, T, is2D=True,symmetryCoupling=True)
ads_model_D2 = partitionmodel(ω_hind_rot_DD, ω_harm_DD, ω_harm_CuDD, ω_ax_DD, ω_lat_DD, ze_harm_HH, inertia_harm_DD, I_D, T, is2D=True,symmetryCoupling=True)
ads_model_T2 = partitionmodel(ω_hind_rot_TT, ω_harm_TT, ω_harm_CuTT, ω_ax_TT, ω_lat_TT, ze_harm_HH, inertia_harm_TT, I_T, T, is2D=True,symmetryCoupling=True)

ads_model_HD = partitionmodel(ω_hind_rot_HD, ω_harm_HD, ω_harm_CuHD, ω_ax_HD, ω_lat_HD, ze_harm_HH, inertia_harm_HD, I_H, T, is2D=True,symmetryCoupling=False)
ads_model_HT = partitionmodel(ω_hind_rot_HT, ω_harm_HT, ω_harm_CuHT, ω_ax_HT, ω_lat_HT, ze_harm_HH, inertia_harm_HT, I_H, T, is2D=True,symmetryCoupling=False)
ads_model_DT = partitionmodel(ω_hind_rot_DT, ω_harm_DT, ω_harm_CuDT, ω_ax_DT, ω_lat_DT, ze_harm_HH, inertia_harm_DT, I_H, T, is2D=True,symmetryCoupling=False)
        


# # Calculation of the adsorption constant of $H_{2}$ isotopologues

# In[5]:


q_ads_model_H2 = ads_model_H2[0]*ads_model_H2[1]*ads_model_H2[2]*ads_model_H2[3]*ads_model_H2[4]*ads_model_H2[5]
q_ads_model_D2 = ads_model_D2[0]*ads_model_D2[1]*ads_model_D2[2]*ads_model_D2[3]*ads_model_D2[4]*ads_model_D2[5]
q_ads_model_T2 = ads_model_T2[0]*ads_model_T2[1]*ads_model_T2[2]*ads_model_T2[3]*ads_model_T2[4]*ads_model_T2[5]
q_ads_model_HD = ads_model_HD[0]*ads_model_HD[1]*ads_model_HD[2]*ads_model_HD[3]*ads_model_HD[4]*ads_model_HD[5]
q_ads_model_HT = ads_model_HT[0]*ads_model_HT[1]*ads_model_HT[2]*ads_model_HT[3]*ads_model_HT[4]*ads_model_HT[5]
q_ads_model_DT = ads_model_DT[0]*ads_model_DT[1]*ads_model_DT[2]*ads_model_DT[3]*ads_model_DT[4]*ads_model_DT[5]


q_gasphase_H2 = gasphase_H2[0]*gasphase_H2[1]*gasphase_H2[2]
q_gasphase_D2 = gasphase_D2[0]*gasphase_D2[1]*gasphase_D2[2]
q_gasphase_T2 = gasphase_T2[0]*gasphase_T2[1]*gasphase_T2[2]
q_gasphase_HD = gasphase_HD[0]*gasphase_HD[1]*gasphase_HD[2]
q_gasphase_HT = gasphase_HT[0]*gasphase_HT[1]*gasphase_HT[2]
q_gasphase_DT = gasphase_DT[0]*gasphase_DT[1]*gasphase_DT[2]


kad_H2 = q_ads_model_H2/q_gasphase_H2
kad_D2 = q_ads_model_D2/q_gasphase_D2
kad_T2 = q_ads_model_T2/q_gasphase_T2


kad_HD = q_ads_model_HD/q_gasphase_HD
kad_HT = q_ads_model_HT/q_gasphase_HT
kad_DT = q_ads_model_DT/q_gasphase_DT


# In[ ]:





# # Separation factor between D2 and H2 at T=80 K ...

# \begin{equation}
#     \alpha_{D_{2}/H_{2}}=\frac{K_{ad,D_{2}}}{K_{ad,H_{2}}}
# \end{equation}
# 
# 

# In[6]:


𝛼_d2h2 = kad_D2 / kad_H2
print('--------Table of Values 𝛼 = D2/H2 -------------')
for i in range(1,24):
    #print("T={:3d}       𝛼={:8f} ".format(10*i,𝛼_t2d2[i-1]))
    print("T={:3d}       𝛼={:8f} ".format(10*(7+i),𝛼_d2h2[i-1]))
    
#𝛼_hdh2 = kad_HD / kad_H2
#print('--------Table of Values 𝛼 = HD/H2 -------------')
#for i in range(1,24):
    #print("T={:3d}       𝛼={:8f} ".format(10*i,𝛼_t2d2[i-1]))
  #  print("T={:3d}       𝛼={:8f} ".format(10*(7+i),𝛼_hdh2[i-1]))
    


# # Separation factor between T2 and D2 at T=80 K ...

# \begin{equation}
#     \alpha_{T_{2}/D_{2}}=\frac{K_{ad,T_{2}}}{K_{ad,D_{2}}}
# \end{equation}
# 

# In[7]:


𝛼_t2d2 = kad_T2 / kad_D2
print('-----------------Table of Values  𝛼 = T2/D2  -------------')
for i in range(1,24):
    #print("T={:3d}       𝛼={:8f} ".format(10*i,𝛼_t2d2[i-1]))
    print("T={:3d}       𝛼={:8f} ".format(10*(7+i),𝛼_t2d2[i-1]))
    
#𝛼_dtd2 = kad_DT / kad_D2
#print('--------Table of Values 𝛼 = DT/D2 -------------')
#for i in range(1,24):
    #print("T={:3d}       𝛼={:8f} ".format(10*i,𝛼_t2d2[i-1]))
    #print("T={:3d}       𝛼={:8f} ".format(10*(7+i),𝛼_dtd2[i-1]))


# In[8]:


𝛼_hth2 = kad_HT / kad_H2
print('--------Table of Values 𝛼 = HT/H2 -------------')
for i in range(1,24):
    #print("T={:3d}       𝛼={:8f} ".format(10*i,𝛼_t2d2[i-1]))
    print("T={:3d}       𝛼={:8f} ".format(10*(7+i),𝛼_hth2[i-1]))


# In[9]:


𝛼_dtd2 = kad_DT / kad_D2
print('--------Table of Values 𝛼 = DT/D2 -------------')
for i in range(1,24):
    #print("T={:3d}       𝛼={:8f} ".format(10*i,𝛼_t2d2[i-1]))
    print("T={:3d}       𝛼={:8f} ".format(10*(7+i),𝛼_dtd2[i-1]))


# In[10]:


𝛼_hdh2 = kad_HD / kad_H2
print('--------Table of Values 𝛼 = HD/H2 -------------')
for i in range(1,24):
    #print("T={:3d}       𝛼={:8f} ".format(10*i,𝛼_t2d2[i-1]))
    print("T={:3d}       𝛼={:8f} ".format(10*(7+i),𝛼_hdh2[i-1]))
    


# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:





# In[ ]:




